/*
 * Copyright (c) 2011-2014 BlackBerry Limited.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import bb.cascades 1.2

TabbedPane {
        
    function checkFirstRun() {
        var first = _app.getValueFor("firstLaunch", "false");
        console.log("Checking first launch: " + first);
        if (first == "false")
        {
            var sheet = tutorialSheet.createObject(this);
            sheet.open();
            _app.saveValueFor("firstLaunch", "true");
        }
    }

    onCreationCompleted: {
        checkFirstRun()
    }
    
    id: tabPane
    objectName: "m_tabPane"
    activeTab: editorTab
    showTabsOnActionBar: false   
    
    onPeekStarted: {
        settingsSheet.close();
        helpSheet.close();
        tutorialSheet.close();
    }
    
    onPeekEnded: {
        settingsSheet.close();
        helpSheet.close();
        tutorialSheet.close();
    } 
    
    Menu.definition: MenuDefinition {
        actions: [
            ActionItem {
                title: qsTr("Settings") + Retranslate.onLocaleOrLanguageChanged
                imageSource: "/img/common/ico/ico_opt.png"                    
                shortcuts: [ Shortcut { key: "E" }, Shortcut { key: "e" }]
                onTriggered: {
                    var sheet = settingsSheet.createObject();
                    sheet.open();
                }
            },
            ActionItem {
                title:  qsTr("Help") + Retranslate.onLocaleOrLanguageChanged
                imageSource: "/img/common/ico/ico_help.png"
                shortcuts: [ Shortcut { key: "H" }, Shortcut { key: "h" }]
                onTriggered: {
                    var sheet = helpSheet.createObject();
                    sheet.open();
                }
            }            
        ]
    }
    
    attachedObjects: [
        ComponentDefinition {
            id: settingsSheet
            source: "pages/SettingsPage.qml"
            objectName: "objSettingsSheet"
        }
        ,
        ComponentDefinition {
            id: helpSheet
            source: "pages/HelpPage.qml"
            objectName: "objHelpSheet"
        }
        ,
        ComponentDefinition {
            id: tutorialSheet
            source: "pages/TutorialPage.qml"
            objectName: "objTutorialSheet"
        }
    ]

    Tab {
        id: editorTab
        title: qsTr("Editor") + Retranslate.onLocaleOrLanguageChanged
        objectName: ("m_editorTab")
        shortcuts: [ Shortcut { key: "E" }, Shortcut { key: "e" }]
        delegate: Delegate {
            id: editorPage
            source: "tabs/EditorTab.qml"
        }
        
        imageSource: "/img/tabs/editorTab.png"
        delegateActivationPolicy: TabDelegateActivationPolicy.ActivateImmediately
    }
    Tab {
        id: recentTab
        title: qsTr("Recent") + Retranslate.onLocaleOrLanguageChanged
        objectName: ("m_recentFiles")
        shortcuts: [ Shortcut { key: "R" }, Shortcut { key: "r" }]
        delegate: Delegate {
            id: recentPage
            source: "tabs/RecentTab.qml"
        }
        
        imageSource: "/img/tabs/recentTab.png"
        delegateActivationPolicy: TabDelegateActivationPolicy.ActivateImmediately
    }
}
/*
 * inmain on creation Qt.tabPane = tabPaneId;  Qt.tab1 = tab1;
 * onclosed
 * Qt.tabPane.setelectedTab(Qt.tab1);
 */