import bb.cascades 1.3
import bb.system 1.0
import bb.cascades.pickers 1.0
import my.timer 1.0

Page {
    id: editorTabPage
    
    property int lastFound: 0
    property string lastFoundString: ""
    
    onCreationCompleted: {
        updateSettings();
        _app.invokeRefresh.connect(onInvokeRefresh);
    }
    
    function onInvokeRefresh(){
        updateSettings();
    }

    
    function updateSettings() {
        var size = parseInt(_app.getValueFor("fontSize", "2"));
        console.log("Updating font size: " + size);
        if (size == 0)
            editorData.textStyle.fontSize = FontSize.XSmall;
        else if (size == 1)
            editorData.textStyle.fontSize = FontSize.Small;
        else if (size == 2)
            editorData.textStyle.fontSize = FontSize.Medium;
        else if (size == 3)
            editorData.textStyle.fontSize = FontSize.Large;
        else if (size == 4)
            editorData.textStyle.fontSize = FontSize.XLarge;
        else 
            editorData.textStyle.fontSize = FontSize.Medium;
        
        var state = _app.getValueFor("dark", "true");
        console.log("Gathering theme setting: " + state);
        if (state=="true") {
            Application.themeSupport.setVisualStyle(VisualStyle.Dark);
        }       
        else {
            Application.themeSupport.setVisualStyle(VisualStyle.Bright);
        }
        
        timer.start();
    }
  
    titleBar: TitleBar {
        id: appTitle
        objectName: "m_appTitle"
        title: qsTr("New File") + Retranslate.onLocaleOrLanguageChanged    
    }
    
    Container {
        
        horizontalAlignment: HorizontalAlignment.Fill
        
        ScrollView {
            
            scrollViewProperties.scrollMode: ScrollMode.Vertical
            id: scrollView            
            
            Container {
                verticalAlignment: VerticalAlignment.Fill
                horizontalAlignment: HorizontalAlignment.Fill
                
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 1
                }
                
                TextArea {
                    
                    id: editorData
                    objectName: "m_editorData"
                    inputMode: TextAreaInputMode.Text
                    input.submitKey: SubmitKey.None
                    textFormat: TextFormat.Plain
                    
                    verticalAlignment: VerticalAlignment.Fill
                    horizontalAlignment: HorizontalAlignment.Fill
                    
                    onFocusedChanged: {
                        updateSettings();
                    }
                    
                    onTextChanging: {
                        
                        if (editorData.text != "" && findButton.enabled == false)
                        {
                            findButton.enabled = true;
                            //clipboardButton.enabled = true;
                        }   
                        if (!revertButton.enabled && _app.getSaved())
                        {
                            saveNowButton.enabled = true;
                            revertButton.enabled = true;
                        }
                        //console.log("Status: " + _app.getSaved() + ":" + _app.getName());
                    }
                                        
                    //editor.onCursorPositionChanged: {
                    //    footerLabel.text = qsTr("Characters") + Retranslate.onLocaleOrLanguageChanged + ": " + editorData.editor.cursorPosition + qsTr(" of ") + Retranslate.onLocaleOrLanguageChanged + editorData.text.length;                        
                    //}
                    focusHighlightEnabled: true
                    textStyle.textAlign: TextAlign.Left
                    
                }                
            }
        }
        Label {
            id: footerLabel
            text: qsTr("Characters") + Retranslate.onLocaleOrLanguageChanged + ": 0" + qsTr(" of ") + Retranslate.onLocaleOrLanguageChanged + "0"
            horizontalAlignment: HorizontalAlignment.Right
        }
    }
    
    attachedObjects: [
        FilePicker {
            id: filePicker
            type : FileType.Other            
            title : qsTr("Select File") + Retranslate.onLocaleOrLanguageChanged
            directories : _app.getPath()
            viewMode: FilePickerViewMode.ListView
            
            allowOverwrite: true
            onPickerOpened: {
                filePicker.allowOverwrite = true;
            }
            onAllowOverwriteChanged: {
                console.log("Overwrite changed : " + allowOverwrite)
            }
            onError: {
                toast.body = qsTr("An error occurred accessing the file") + Retranslate.onLocaleOrLanguageChanged;
                toast.show();
            }
            onFileSelected : {
                console.log("FileSelected signal received : " + selectedFiles);
                filePicker.allowOverwrite = true;
                
                if (mode == FilePickerMode.Saver)
                {
                    selectedFiles.forEach(function(entry) {
                            _app.saveText(entry, editorData.text);
                            var val = _app.getValueFor("recent", "");
                            if (val.indexOf(entry + "\n")== -1)                
                                _app.saveValueFor("recent", _app.getValueFor("recent", "") + entry + "\n");
                    });
                }
                else if (mode == FilePickerMode.Picker)
                {
                    selectedFiles.forEach(function(entry) {
                            _app.loadText(entry);
                            var val = _app.getValueFor("recent", "");
                            if (val.indexOf(entry + "\n")== -1)                
                                _app.saveValueFor("recent", _app.getValueFor("recent", "") + entry + "\n");
                    });
                }
                _app.onInvokeRefresh();
            }
            objectName: "m_filePicker"
        },
        SystemToast {
            id: toast
        },
        Invocation {
            id: linkInvocation
            property bool auto_trigger: false
            query {
                uri: "http://www.filearchivehaven.com"
                onUriChanged: {
                    linkInvocation.query.updateQuery();
                }
            }
            
            onArmed: {
                // don't auto-trigger on initial setup
                if (auto_trigger)
                    trigger("bb.action.OPEN");
                auto_trigger = true; // allow re-arming to auto-trigger
            }
        }
        ,
        QTimer{
            id: timer
            //set singleshot property if requireds
            singleShot: false
            //set interval
            interval: 250
            
            onTimeout:{
                //do some stuff
                footerLabel.text = qsTr("Characters") + Retranslate.onLocaleOrLanguageChanged + ": " + editorData.editor.cursorPosition + qsTr(" of ") + Retranslate.onLocaleOrLanguageChanged + editorData.text.length;
                
            }
        }
    ]
    
    actions: [
        // context items
        //         title: qsTr("New File") + Retranslate.onLocaleOrLanguageChanged
        ActionItem {
            title: qsTr("Open File") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_open.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            shortcuts: [
                Shortcut {
                    key: "O"
                },
                Shortcut {
                    key: "o"
                }
            ]
            attachedObjects: [
                SystemDialog {
                    id: openDialog
                    title: qsTr("Open File") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Are you sure?  You're current changes will not be saved.") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {
                        if (openDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            filePicker.mode = FilePickerMode.Picker;
                            title = qsTr("Select File") + Retranslate.onLocaleOrLanguageChanged
                            filePicker.directories = _app.getPath();
                            filePicker.open();
                        }
                    }
                }
            ]         
            onTriggered: {
                if (revertButton.enabled)
                {
                    openDialog.show();
                }
                else 
                {
                    filePicker.mode = FilePickerMode.Picker;
                    title = qsTr("Select File") + Retranslate.onLocaleOrLanguageChanged
                    filePicker.directories = _app.getPath();
                    filePicker.open();
                }
                    
            }
        },
        ActionItem {
            title: qsTr("New File") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_new.png"
            ActionBar.placement: ActionBarPlacement.Signature
            
            attachedObjects: [
                SystemDialog {
                    id: newDialog
                    title: qsTr("New File") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Are you sure?  You're current changes will not be saved.") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {
                        if (newDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            lastFound = 0;
                            lastFoundString = "";
                            editorData.text = "";
                            _app.clearValues();
                            //clipboardButton.setEnabled(false);
                            revertButton.setEnabled(false);
                            saveNowButton.setEnabled(false)
                            findButton.setEnabled(false);
                            findNextButton.setEnabled(false);
                            appTitle.title = qsTr("New File") + Retranslate.onLocaleOrLanguageChanged;
                        }
                    }
                }
            ]
            
            onTriggered: {
                if (revertButton.enabled)
                {
                    newDialog.show();
                }
                else 
                {
                    lastFound = 0;
                    lastFoundString = "";
                    editorData.text = "";
                    _app.clearValues();
                    revertButton.setEnabled(false);
                    saveNowButton.setEnabled(false)
                    findButton.setEnabled(false);
                    findNextButton.setEnabled(false);
                    appTitle.title = qsTr("New File") + Retranslate.onLocaleOrLanguageChanged;
                }
            }
        },
        ActionItem {
            objectName: "m_revertButton"
            id: revertButton
            title: qsTr("Revert File") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_reload.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            attachedObjects: [
                SystemDialog {
                    id: revertDialog
                    title: qsTr("Revert File") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Are you sure?  You're current changes will not be saved.") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {
                        if (revertDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            lastFound = 0;
                            lastFoundString = "";
                            findButton.setEnabled(true);
                            findNextButton.setEnabled(false);
                            revertButton.setEnabled(false);
                            saveNowButton.setEnabled(false)
                            _app.loadText(_app.getName());
                        }
                    }
                }
            ]
            
            onTriggered: {
                revertDialog.show();
            }
            enabled: false
            shortcuts: [
                Shortcut {
                    key: "Z"
                },
                Shortcut {
                    key: "z"
                }
            ]            
        },
        ActionItem {
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: qsTr("Share") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "asset:///img/common/ico/ico_share.png"
            shortcuts: [
                Shortcut {
                    key: "E"
                },
                Shortcut {
                    key: "e"
                }
            ]
            attachedObjects: [      
                Invocation {
                    id: invokeShare
                    query: InvokeQuery {
                        id: shareQuery
                    }
                    onArmed:{                    
                        if (shareQuery.data!="" || shareQuery.uri!="")
                            trigger("bb.action.SHARE");
                    }
                }
            ]
            onTriggered: {
                if (_app.getName() == "")
                {
                    toast.body = qsTr("You must save the file before you can share it.") + Retranslate.onLocaleOrLanguageChanged;
                    toast.show();
                    return;
                }
                if (revertButton.enabled)
                {
                    toast.body = qsTr("Cannot share the file, you have unsaved changes.") + Retranslate.onLocaleOrLanguageChanged;
                    toast.show();
                    return;
                }
                
                invokeShare.trigger("bb.action.SHARE");
                shareQuery.mimeType= "text/plain"
                shareQuery.uri = "file://" + _app.getName();
                shareQuery.updateQuery();
            
            }
        },
        ActionItem {
            title: qsTr("Report Bug") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_bug.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            
            attachedObjects: [
                SystemDialog {
                    id: bugDialog
                    title: qsTr("Report Bug") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Tip:  When submitting a bug report, try to include steps to reproduce the problem.") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {
                        if (bugDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            bugQuery.trigger("bb.action.SENDEMAIL");
                        }
                    }
                },
                Invocation {
                    id: bugQuery
                    query {
                        mimeType: "text/plain"
                        invokeTargetId: "sys.pim.uib.email.hybridcomposer"
                        uri: "mailto:support@filearchivehaven.com?subject=Bug%20Report%20%20" + Application.applicationVersion
                    }
                }
            ]
            
            onTriggered: {
                bugDialog.show();
            }
        },
        ActionItem {
            title: qsTr("Known Bugs") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_knownbug.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [
                Shortcut {
                    key: "K"
                },
                Shortcut {
                    key: "k"
                }
            ]
            
            onTriggered: {
                _downloadManager.getOnlineData("http://www.filearchivehaven.com/data/TextEditHD/bugs")
            }
        },
        ActionItem {
            title: qsTr("Write Review") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_review.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [
                Shortcut {
                    key: "R"
                },
                Shortcut {
                    key: "r"
                }
            ]
            
            attachedObjects: [
                SystemDialog {
                    id: reviewDialog
                    title: qsTr("Write Review") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("I need your help to keep making this application great!  If you experience any issues, please try contacting me before leaving a negative review (so I can fix the problem).  Please be honest in your review, but keep in mind that positive reviews means more users (and ultimately more improvements).") + Retranslate.onLocaleOrLanguageChanged
                    onFinished: {
                        if (reviewDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            reviewQuery.trigger("bb.action.OPEN")
                        }
                    }
                },
                Invocation {
                    id: reviewQuery
                    query {
                        mimeType: "text/html"
                        invokeTargetId: "sys.appworld"
                        uri: "appworld://content/25488873"
                    }
                }
            ]
            
            onTriggered: {
                reviewDialog.show()
            }
        },
        ActionItem {
            id: findButton
            title: qsTr("Find") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_find.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            attachedObjects: [
                SystemPrompt {
                    id: findDialog
                    title: qsTr("Find") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Enter the text to search for.") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {                        
                        if (findDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            var find = editorData.text.search(findDialog.inputFieldTextEntry());
                            if (find != -1)
                            {
                                lastFoundString = findDialog.inputFieldTextEntry();
                                lastFound = find;
                                toast.body = qsTr("Found at character ") + Retranslate.onLocaleOrLanguageChanged + find;
                                toast.show();
                                //editorData.editor.cursorPosition = lastFound;
                                editorData.editor.setSelection(lastFound, lastFound + findDialog.inputFieldTextEntry().length);
                                editorData.requestFocus();
                                if (find < editorData.text.length)
                                    findNextButton.enabled = true;
                                else 
                                    findNextButton.enabled = false;
                            }
                            else 
                            {
                                lastFoundString = findDialog.inputFieldTextEntry();
                                toast.body = qsTr("Can't find on page.") + Retranslate.onLocaleOrLanguageChanged
                                toast.show();
                                lastFound = 0;
                                findNextButton.enabled = false;
                            }
                        }
                    }
                    returnKeyAction: SystemUiReturnKeyAction.Go
                }
            ]
            enabled: false
            shortcuts: [
                Shortcut {
                    key: "F"
                },
                Shortcut {
                    key: "f"
                }
            ]
            
            onTriggered: {
                findDialog.show();
            }
        },
        ActionItem {
            id: findNextButton
            enabled: false
            title: qsTr("Find Next") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_find.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            attachedObjects: [
                SystemDialog {
                    id: findNextDialog
                    title: qsTr("Find Next") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Find next '") + Retranslate.onLocaleOrLanguageChanged + lastFoundString + "'";
                    
                    onFinished: {                        
                        if (findDialog.result == SystemUiResult.ConfirmButtonSelection) {
                        
                        }
                    }
                    returnKeyAction: SystemUiReturnKeyAction.Go
                }
            ]
            shortcuts: [
                Shortcut {
                    key: "N"
                },
                Shortcut {
                    key: "n"
                }
            ]
            
            onTriggered: {
                //findNextDialog.show();
                var find = editorData.text.substring(lastFound+1, editorData.text.length).search(lastFoundString);
                if (find != -1)
                {
                    lastFound = lastFound + 1 + find;
                    toast.body = qsTr("Found at character ") + Retranslate.onLocaleOrLanguageChanged + lastFound;
                    //editorData.editor.cursorPosition = lastFound;
                    editorData.editor.setSelection(lastFound, lastFound + lastFoundString.length);
                    editorData.requestFocus();
                    toast.show();
                    if (find < editorData.text.length)
                        findNextButton.enabled = true;
                    else 
                        findNextButton.enabled = false;
                }
                else 
                {
                    toast.body = qsTr("Can't find next on page.") + Retranslate.onLocaleOrLanguageChanged
                    toast.show();
                    lastFound = 0;
                    lastFoundString = "";
                    findNextButton.enabled = false;
                }
            }
        },
        ActionItem {
            title: qsTr("Copy to Clipboard") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_clip.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            id: clipboardButton
            enabled: true
            shortcuts: [
                Shortcut {
                    key: "C"
                },
                Shortcut {
                    key: "c"
                }
            ]
            
            onTriggered: {
                _app.copyText(editorData.text);
                toast.body = qsTr("Copied to Clipboard") + Retranslate.onLocaleOrLanguageChanged;
                toast.show();
            }
        }
        ,
        ActionItem {
            title: qsTr("Save As") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_save.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [
                Shortcut {
                    key: "A"
                },
                Shortcut {
                    key: "a"
                }
            ]
            
            onTriggered: {
                filePicker.mode = FilePickerMode.Saver;
                filePicker.allowOverwrite = true;
                filePicker.defaultSaveFileNames = _app.getName()
                filePicker.directories = _app.getPath();
                title : qsTr("Save As") + Retranslate.onLocaleOrLanguageChanged
                filePicker.open();
            }
        }
        ,
        
        ActionItem {
            title: qsTr("Save") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_savenow.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            enabled: false
            id: saveNowButton
            shortcuts: [
                Shortcut {
                    key: "S"
                },
                Shortcut {
                    key: "s"
                }
            ]
            
            attachedObjects: [
                SystemDialog {
                    id: saveDialog
                    title: qsTr("Overwrite") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Overwrite this file?") + Retranslate.onLocaleOrLanguageChanged
                    onFinished: {
                        if (saveDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            _app.saveText(_app.getName(), editorData.text)
                        }
                    }
                }
            ]
            
            onTriggered: {
                saveDialog.show()
            }
        }
    ]
}
