import bb.cascades 1.3
import bb.system 1.2

Page {
    id: recentTabPage
    
    onCreationCompleted: {
        updateSettings();
        Qt.arrayModel = arrayModel;
        arrayModel.mfReload();
        _app.invokeRefresh.connect(onInvokeRefresh);
    }
    
    function onInvokeRefresh(){
        Qt.arrayModel.mfReload();
    }
    
    function updateSettings() {
        
        var state = _app.getValueFor("dark", "true");
        console.log("Gathering theme setting: " + state);
        if (state=="true") {
            Application.themeSupport.setVisualStyle(VisualStyle.Dark);
        }       
        else {
            Application.themeSupport.setVisualStyle(VisualStyle.Bright);
        }
    }
    
    titleBar: TitleBar {
        id: recentTitle
        objectName: "m_recentTitle"
        title: qsTr("Recent Files") + Retranslate.onLocaleOrLanguageChanged    
    }
    
    Container {
        horizontalAlignment: HorizontalAlignment.Fill
        
        
        Container {
            verticalAlignment: VerticalAlignment.Fill
            horizontalAlignment: HorizontalAlignment.Fill
            
            layoutProperties: StackLayoutProperties {
                spaceQuota: 1
            }
            ListView {
                id: recentFiles
                
                dataModel: ArrayDataModel {
                    id: arrayModel
                    property alias myArrayModel: arrayModel
                    function mfLoadItem(item){
                        console.log("Loading ... " + item);
                        tabPane.activeTab = editorTab;
                        _app.loadRecent(item);
                    }
                    function mfClearSavedItems(){
                        _app.saveValueFor("recent", "");
                        arrayModel.clear();
                    }
                    function mfRemoveSavedItem(item){
                        _app.saveValueFor("recent", _app.getValueFor("recent", "").replace(item, ""))
                        mfReload();
                    }
                    function mfAddNewItem(newName){
                        _app.saveValueFor("recent", _app.getValueFor("recent", "") + newName + "\n")
                        mfReload();
                    }                    
                    function mfReload()
                    {
                        arrayModel.clear();
                        var data = _app.getValueFor("recent", "").split("\n");
                        var newSavedItems = "";
                        _app.saveValueFor("recent", "");
                        // clear all records
                        for ( var i = 0; i < data.length; i++ ) {
                            if(data[i] != "") {
                                data[i].replace("\n","");
                                // skip empty items
                                arrayModel.append(data[i]);
                                newSavedItems = newSavedItems + data[i] + "\n";
                            }
                        }
                        _app.saveValueFor("recent", newSavedItems);                    
                    
                    }
                
                }
                
                onTriggered: {
                    clearSelection();
                    select(0, true);
                }                 
                
                listItemComponents: [
                    ListItemComponent {
                        
                        type: ""
                        
                        CustomListItem {
                            
                            dividerVisible: true
                            highlightAppearance: HighlightAppearance.Full
                            
                            Container {                                    
                                layout: StackLayout {
                                    orientation: LayoutOrientation.LeftToRight
                                }
                                
                                ImageView {
                                    imageSource: "asset:///img/common/ico/ico_file.png"
                                }
                                
                                Label {
                                    id: lblDescription                                        
                                    text: qsTr(ListItemData).split("\t")[0].trim()                                         
                                }
                            }
                            
                            contextActions: [
                                // Add a set of four actions to the context menu for
                                // a list item
                                ActionSet {
                                    id: actionSet
                                    ActionItem {
                                        title: qsTr("Remove Item") + Retranslate.onLocaleOrLanguageChanged
                                        imageSource: "/img/common/ico/ico_remove.png"
                                        shortcuts: [
                                            Shortcut {
                                                key: "D"
                                            },
                                            Shortcut {
                                                key: "d"
                                            }
                                        ]
                                        onTriggered: {
                                            var index =  Qt.arrayModel.indexOf(ListItemData.toString())
                                            if(index != -1){
                                                Qt.arrayModel.removeAt(index)
                                                Qt.arrayModel.mfRemoveSavedItem(ListItemData.toString())
                                            }
                                        }
                                    }
                                    ActionItem {
                                        title: qsTr("Open Item") + Retranslate.onLocaleOrLanguageChanged
                                        imageSource: "/img/common/ico/ico_reopen.png"
                                        shortcuts: [
                                            Shortcut {
                                                key: "O"
                                            },
                                            Shortcut {
                                                key: "o"
                                            }
                                        ]
                                        onTriggered: {
                                            Qt.arrayModel.mfLoadItem(ListItemData.toString());
                                        
                                        }
                                    }
                                    ActionItem {
                                        title: qsTr("Clear All Items") + Retranslate.onLocaleOrLanguageChanged
                                        imageSource: "/img/common/ico/ico_removeall.png"
                                        shortcuts: [
                                            Shortcut {
                                                key: "C"
                                            },
                                            Shortcut {
                                                key: "c"
                                            }
                                        ]
                                        onTriggered: {
                                            Qt.arrayModel.mfClearSavedItems();
                                        }
                                    }
                                }
                            ]
                        }
                    }
                ]
                onCreationCompleted: {
                    var data = _app.getValueFor("recent", "").split("\n")
                    var newSavedItems = ""
                    _app.saveValueFor("recent", "")  // clear all records
                    for ( var i = 0; i < data.length; i++ ) {
                        if(data[i] != "") {
                            data[i].replace("\n","");
                            // skip empty items
                            arrayModel.append(data[i]);
                            newSavedItems = newSavedItems + data[i] + "\n";
                        }
                    }
                    _app.saveValueFor("recent", newSavedItems);                    
                }
            
            }
        }    
    }
    attachedObjects: [
        
        SystemToast {
            id: toast
        },
        Invocation {
            id: linkInvocation
            property bool auto_trigger: false
            query {
                uri: "http://www.filearchivehaven.com"
                onUriChanged: {
                    linkInvocation.query.updateQuery();
                }
            }
            
            onArmed: {
                // don't auto-trigger on initial setup
                if (auto_trigger)
                    trigger("bb.action.OPEN");
                auto_trigger = true; // allow re-arming to auto-trigger
            }
        }
    ]
    actions: [
        // context items
        //         title: qsTr("New File") + Retranslate.onLocaleOrLanguageChanged
        
        ActionItem {
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: qsTr("Share") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "asset:///img/common/ico/ico_share.png"
            shortcuts: [
                Shortcut {
                    key: "E"
                },
                Shortcut {
                    key: "e"
                }
            ]
            attachedObjects: [      
                Invocation {
                    id: invokeShare
                    query: InvokeQuery {
                        id: shareQuery
                    }
                    onArmed:{                    
                        if (shareQuery.data!="" || shareQuery.uri!="")
                            trigger("bb.action.SHARE");
                    }
                }
            ]
            onTriggered: {
                invokeShare.trigger("bb.action.SHARE");
                shareQuery.data = qsTr("Check out TextEditHD in BlackBerry World http://appworld.blackberry.com/webstore/content/25488873") + Retranslate.onLocaleOrLanguageChanged;
                shareQuery.updateQuery();            
            }
        },
        ActionItem {
            title: qsTr("Report Bug") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_bug.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            
            attachedObjects: [
                SystemDialog {
                    id: bugDialog
                    title: qsTr("Report Bug") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("Tip:  When submitting a bug report, try to include steps to reproduce the problem.") + Retranslate.onLocaleOrLanguageChanged
                    
                    onFinished: {
                        if (bugDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            bugQuery.trigger("bb.action.SENDEMAIL");
                        }
                    }
                },
                Invocation {
                    id: bugQuery
                    query {
                        mimeType: "text/plain"
                        invokeTargetId: "sys.pim.uib.email.hybridcomposer"
                        uri: "mailto:support@filearchivehaven.com?subject=Bug%20Report%20%20" + Application.applicationVersion
                    }
                }
            ]
            
            onTriggered: {
                bugDialog.show();
            }
        },
        ActionItem {
            title: qsTr("Known Bugs") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_knownbug.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [
                Shortcut {
                    key: "K"
                },
                Shortcut {
                    key: "k"
                }
            ]
            
            onTriggered: {
                _downloadManager.getOnlineData("http://www.filearchivehaven.com/data/TextEditHD/bugs")
            }
        },
        ActionItem {
            title: qsTr("Write Review") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_review.png"
            ActionBar.placement: ActionBarPlacement.InOverflow
            shortcuts: [
                Shortcut {
                    key: "R"
                },
                Shortcut {
                    key: "r"
                }
            ]
            
            attachedObjects: [
                SystemDialog {
                    id: reviewDialog
                    title: qsTr("Write Review") + Retranslate.onLocaleOrLanguageChanged
                    body: qsTr("I need your help to keep making this application great!  If you experience any issues, please try contacting me before leaving a negative review (so I can fix the problem).  Please be honest in your review, but keep in mind that positive reviews means more users (and ultimately more improvements).") + Retranslate.onLocaleOrLanguageChanged
                    onFinished: {
                        if (reviewDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            reviewQuery.trigger("bb.action.OPEN")
                        }
                    }
                },
                Invocation {
                    id: reviewQuery
                    query {
                        mimeType: "text/html"
                        invokeTargetId: "sys.appworld"
                        uri: "appworld://content/25488873"
                    }
                }
            ]
            
            onTriggered: {
                reviewDialog.show()
            }
        },
        ActionItem {
            title: qsTr("Clear All Items") + Retranslate.onLocaleOrLanguageChanged
            imageSource: "/img/common/ico/ico_removeall.png"
            ActionBar.placement: ActionBarPlacement.Signature
            shortcuts: [
                Shortcut {
                    key: "C"
                },
                Shortcut {
                    key: "c"
                }
            ]
            onTriggered: {
                Qt.arrayModel.mfClearSavedItems();
            }
        }
    ]
}
